# coding=utf-8
from __future__ import absolute_import, print_function

from suanpan import api


def getAccessKey():
    return api.affinity.get("/oss/ak")


def getToken():
    return api.affinity.get("/oss/token")
